/*
 * Copyright (c) 2014 Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.openjdk.bench.java.lang;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.OutputTimeUnit;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;

import java.util.concurrent.TimeUnit;

@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@State(Scope.Thread)
public class StringUpperLower {

    private String strLowerCase;
    private String strMixedCase;
    private String strUpperCase;

    @Setup
    public void setup() {
        strLowerCase = "this is a simple string with different cases";
        strMixedCase = "This IS a SImpLE strING wITh DiFfErEnT CASeS";
        strUpperCase = "THIS IS A SIMPLE STRING WITH DIFFERENT CASES";
    }

    @Benchmark
    public String lowerToLower() {
        return strLowerCase.toLowerCase();
    }

    @Benchmark
    public String mixedToLower() {
        return strMixedCase.toLowerCase();
    }

    @Benchmark
    public String upperToLower() {
        return strUpperCase.toLowerCase();
    }

    @Benchmark
    public String lowerToUpper() {
        return strLowerCase.toUpperCase();
    }

    @Benchmark
    public String mixedToUpper() {
        return strMixedCase.toUpperCase();
    }

    @Benchmark
    public String upperToUpper() {
        return strUpperCase.toUpperCase();
    }

}
